package com.example.service.serviceImpl;

import com.example.db1.dao.TscanAssortDao;
import com.example.db2.dao.LianZhongDao;
import com.example.service.CollectService;
import com.example.vo.CardInfoVo;
import com.example.vo.Picture;
import com.example.dto.TscanAssortDto;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @ClassName CollectServiceImpl
 * @Description 同步联众数据
 * @Author linjj
 * @Date 2025/6/4 13:12
 * @Version 1.0
 */
@Service
@Slf4j
public class CollectServiceImpl implements CollectService {
    @Value("${savePath}")
    private String savePath;
    @Autowired
    private LianZhongDao lianZhongDao;
    @Autowired
    private TscanAssortDao tscanAssortDao;
    private final ExecutorService executorService = Executors.newFixedThreadPool(2); // 创建一个包含两个线程的线程池

    private final String uniUrl = "http://10.1.1.128";

    @Override
    public void LianZhongConllect() {
        //查询所有联众需要同步数据，未开始状态为0，正在同步为1，完成为2，失败为3，不存在文件为5
        Integer collectNum = lianZhongDao.getCollectNum(5);
        log.info("需要同步: " + collectNum + "条数据");

        // 计算需要轮询的次数
        int totalLoops = (collectNum + 999) / 1000; // 向上取整

        // 使用AtomicInteger来记录当前任务的编号
        AtomicInteger currentTask = new AtomicInteger(0);
        // 使用CountDownLatch来控制任务的执行顺序
        CountDownLatch latch = new CountDownLatch(1);

        // 提交所有任务到线程池
        for (int i = 0; i < totalLoops; i++) {
            final int taskNum = i;
            executorService.submit(() -> {
                try {
                    // 等待前一个任务完成
                    latch.await();
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt(); // 重新设置中断状态
                }
                // 计算当前任务处理的数据范围
                int start = taskNum * 1000;
                int end = Math.min(start + 999, collectNum); // 确保不超过总条数
                //执行任务
                conllect(taskNum, start, end);
                // 通知下一个任务可以开始
                latch.countDown();
            });

            // 每两个任务提交后，释放一个计数器，允许下一个任务开始
            if (currentTask.incrementAndGet() % 2 == 0) {
                latch.countDown();
            }
        }

        // 等待所有任务完成
        try {
            executorService.shutdown();
            if (!executorService.awaitTermination(60, TimeUnit.SECONDS)) {
                executorService.shutdownNow();
            }
        } catch (InterruptedException e) {
            executorService.shutdownNow();
            Thread.currentThread().interrupt();
        }
    }

    /**
     * @description: 同步逻辑
     * @author linjj
     * @date: 2025/6/4 13:33
     */
    private void conllect(int taskNum, int start, int end) {
        log.info("轮询第 " + (taskNum + 1) + " 次，处理数据范围: " + start + " 到 " + end);
        //根据start跟end查询记录保证数据不会重复
        List<CardInfoVo> cardInfoVo = lianZhongDao.getCardInfoVo(start, end);
        //每个患者进行轮询
        for (CardInfoVo list : cardInfoVo) {
            try {
//                //用于存储数码影像数据
//                ArrayList<TscanAssortDto> tscanAssortDtos = new ArrayList<>();
                log.info("当前同步患者id为:" + list.getId());
                //查询患者当前文件信息
                List<Picture> pictures = lianZhongDao.getPicture(list.getId());
                if (ObjectUtils.isEmpty(pictures)) {
                    log.info("当前患者没有文件:" + list.getId());
                    continue;
                }
                //组织cmd命令
                for (Picture picture : pictures) {
                    try {
                        //保存目录
                        String dir = savePath + File.separatorChar + list.getId();
                        //保存地址
                        String savePath = dir + File.separatorChar + picture.getPicname();
                        // 创建File对象
                        File directory = new File(dir);
                        // 判断目录是否存在
                        if (!directory.exists()) {
                            // 目录不存在，创建目录
                            boolean created = directory.mkdirs();
                            if (created) {
                                log.info("目录创建成功：" + dir);
                            } else {
                                log.info("目录创建失败：" + dir);
                            }
                        }
                        //组织cmd命令(D:\js\Debug\Debug\lianzhong.exe为路径,10.1.1.128为联众弟子)
                        String cmd = "D:\\js\\Debug\\Debug\\lianzhong.exe 003 10.1.1.128 " + list.getId() + " " + picture.getPicid() + " " + list.getPatno() + " " + list.getOutdateStr() + " " + picture.getPicname() + " " + savePath + " " + uniUrl + " " + picture.getRotatedegree();
                        log.info("CMD:" + cmd);
                        Process process = Runtime.getRuntime().exec(cmd);//执行命令生成cube
                        process.waitFor();
                    } catch (Exception e) {
                        log.info("患者同步失败:" + list.getId());
                        log.error(e.getMessage(), e);
                    }
//                    //完成后保存文件表
//                    TscanAssortDto tscanAssortDto =new TscanAssortDto();
//                    tscanAssortDto.setPatientId(list.getId());
//                    tscanAssortDto.setAssortId("lz"+picture.getPickind());
//                    tscanAssortDto.setScanPage(picture.getPicname());
//                    tscanAssortDto.setSource("1");
//                    tscanAssortDto.setIsDel("0");
//                    tscanAssortDto.setSourceFlag("lz");
//                    tscanAssortDtos.add(tscanAssortDto);
                }
//                //保存文件，根据返修改改状态成功2失败3
//                if (tscanAssortDao.addTscanAssortDto(tscanAssortDtos)){
//                    lianZhongDao.updateState(list.getId(),2);
//                    log.info("患者同步完成:"+list.getId());
//                }else {
//                    lianZhongDao.updateState(list.getId(),3);
//                    log.info("患者同步失败:"+list.getId());
//                }

            } catch (Exception e) {
                lianZhongDao.updateState(list.getId(), 3);
                log.info("患者同步失败:" + list.getId());
                log.error(e.getMessage(), e);
                continue;
            }
            lianZhongDao.updateState(list.getId(), 2);
            log.info("患者同步完成:" + list.getId());
        }
    }
}
