package com.example.util;

import lombok.SneakyThrows;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

public class MyDateUtil {

    //获取之前的年限
    public static String getBeforeAgeYear(int age) {
        Calendar calBegin = Calendar.getInstance();
        calBegin.setTime(new Date());
        calBegin.add(Calendar.YEAR, -age);
        Date time = calBegin.getTime();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return format.format(time);
    }


    //计算两个日期相差年数
    public static int yearDateDiff(String startDate, Date endDate) {
        Calendar calBegin = Calendar.getInstance(); //获取日历实例
        Calendar calEnd = Calendar.getInstance();
        calBegin.setTime(stringTodate(startDate, "yyyy-MM-dd")); //字符串按照指定格式转化为日期
        calEnd.setTime(endDate);
        return calEnd.get(Calendar.YEAR) - calBegin.get(Calendar.YEAR);
    }

    /**
     * 　　 *字符串的日期格式的计算
     */
    @SneakyThrows
    public static int daysBetween(String smdate, String bdate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Calendar cal = Calendar.getInstance();
        cal.setTime(sdf.parse(smdate));
        long time1 = cal.getTimeInMillis();
        cal.setTime(sdf.parse(bdate));
        long time2 = cal.getTimeInMillis();
        long between_days = (time2 - time1) / (1000 * 3600 * 24);

        return Integer.parseInt(String.valueOf(between_days));
    }


    @SneakyThrows
    public static int daysBetween(Date smdate, Date bdate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(smdate);
        long time1 = cal.getTimeInMillis();
        cal.setTime(bdate);
        long time2 = cal.getTimeInMillis();
        long between_days = (time2 - time1) / (1000 * 3600 * 24);

        return Integer.parseInt(String.valueOf(between_days));
    }


    public static int hoursBetween(Date smdate, Date bdate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(smdate);
        long time1 = cal.getTimeInMillis();
        cal.setTime(bdate);
        long time2 = cal.getTimeInMillis();
        long between_days = (time2 - time1) / (1000 * 3600);

        return Integer.parseInt(String.valueOf(between_days));
    }

    //字符串按照指定格式转化为日期
    public static Date stringTodate(String dateStr, String formatStr) {
        // 如果时间为空则默认当前时间
        Date date = null;
        SimpleDateFormat format = new SimpleDateFormat(formatStr);
        if (dateStr != null && !dateStr.equals("")) {
            String time = "";
            try {
                Date dateTwo = format.parse(dateStr);
                time = format.format(dateTwo);
                date = format.parse(time);
            } catch (ParseException e) {
                e.printStackTrace();
            }

        } else {
            String timeTwo = format.format(new Date());
            try {
                date = format.parse(timeTwo);
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }
        return date;
    }

    @SneakyThrows
    public static String getYearAgoAndAfter(int year, int y) {
        DateFormat df = new SimpleDateFormat("yyyy");
        Date date = df.parse(String.valueOf(year));
        Calendar instance = Calendar.getInstance();
        instance.setTime(date);
        instance.add(Calendar.YEAR, y);
        Date time = instance.getTime();
        DateFormat yyyyMMdd = new SimpleDateFormat("yyyyMMdd");
        return yyyyMMdd.format(time);
    }

    @SneakyThrows
    public static String getyyyyAgoAndAfter(int year, int y) {
        DateFormat df = new SimpleDateFormat("yyyy");
        Date date = df.parse(String.valueOf(year));
        Calendar instance = Calendar.getInstance();
        instance.setTime(date);
        instance.add(Calendar.YEAR, y);
        Date time = instance.getTime();
        return df.format(time);
    }


    @SneakyThrows
    public static boolean isTodayTrade(Date currDate, Date tradeDate) {
        //判断当前时间是否大于当天21：00 ，如果是是在今天内。
        //判断当前时间是否大于昨天21：00，则不在当天内
        Calendar calendar = Calendar.getInstance(Locale.CHINA);
        calendar.setTime(currDate);
        calendar.add(Calendar.DAY_OF_YEAR, -1);
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        String yyyyMMddYestoday = df.format(calendar.getTime());
        String yes21 = yyyyMMddYestoday + " 21:00:00";
        DateFormat yyyyMMddHHmmss = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date yes21Date = yyyyMMddHHmmss.parse(yes21);


        String yyyyMMddToday = df.format(currDate);
        String today21 = yyyyMMddToday + " 21:00:00";
        Date toy21Date = yyyyMMddHHmmss.parse(today21);


        //交易时间小于昨天的21点，一定是昨天开的仓
        if (tradeDate.before(yes21Date)) {
            return false;
        }

        //交易时间大于昨天的21点。交易时间小于今天的21点。且当前时间小于今天的21点。
        if (tradeDate.after(yes21Date) && tradeDate.before(toy21Date) && currDate.before(toy21Date)) {
            return true;
        }

        //交易时间大与今天的21点。
        if (tradeDate.after(toy21Date) && currDate.after(toy21Date)) {
            return true;
        }

        return false;
    }


    public static String dateToString(Date date, String format) {
        DateFormat df = new SimpleDateFormat(format);
        return df.format(date);
    }

    /**
     * 前后相等
     *
     * @param date
     * @param timeStr1
     * @param timeStr2
     * @return
     */
    public static boolean dateBetweenTime1AndTime2(Date date, String timeStr1, String timeStr2) {
        String formatStr = "HH:mm:ss";
        Date date1 = stringTodate(timeStr1, formatStr);
        Date date2 = stringTodate(timeStr2, formatStr);
        Date curdate = stringTodate(dateToString(date, formatStr), formatStr);
        if ((date1.before(curdate) || date1.equals(curdate)) && (curdate.before(date2) || curdate.equals(date2))) {
            return true;
        }
        return false;
    }



    public static boolean dateBetweenTime1AndTime3(Date date, String timeStr1, String timeStr2) {
        String formatStr = "HH:mm:ss";
        Date date1 = stringTodate(timeStr1, formatStr);
        Date date2 = stringTodate(timeStr2, formatStr);
        Date curdate = stringTodate(dateToString(date, formatStr), formatStr);
        if ((date1.before(curdate)  && (curdate.before(date2) || curdate.equals(date2)))) {
            return true;
        }
        return false;
    }


    public static boolean dateBefore(String timeStr1, String timeStr2) {
        String formatStr = "HH:mm:ss";
        Date date1 = stringTodate(timeStr1, formatStr);
        Date date2 = stringTodate(timeStr2, formatStr);
        return date1.before(date2);
    }

}
